CON
    _clkmode = xtal1+pll16x
    _clkfreq = 80_000_000

CON
    LEFT_MARGIN         = 0
    RIGHT_MARGIN        = 0
    TOP_MARGIN          = 0
    BOT_MARGIN          = 0
    INTERLACE           = 0
    C_PRIME             = 30
    CLOCK_STEP          = 250_000
    H_SYNC_PER          = 8
    M_PRIME             = 300
    MIN_V_PORCH_RND     = 3
    MIN_V_BPORCH        = 6
    MIN_VSYNC_BP        = 550
    RB_H_BLANK          = 160
    RB_H_SYNC           = 32
    RB_MIN_V_BLANK      = 460
    RB_V_FPORCH         = 3
    CELL_GRAN_RND       = 8

VAR
    long    V_FIELD_RATE_RQD
    long    H_PIXELS_RND
    long    TOTAL_ACTIVE_PIXELS
    long    V_LINES_RND
    long    H_PERIOD_EST            ' ns
    long    V_SYNC_BP
    long    V_SYNC_RND
    long    V_BACK_PORCH
    long    TOTAL_V_LINES
    long    IDEAL_DUTY_CYCLE
    long    H_BLANK
    long    TOTAL_PIXELS
    long    ACT_PIXEL_FREQ
    long    ACT_H_FREQ
    long    ACT_FIELD_RATE
    long    H_FP
    long    H_SYNC
    long    H_BP
    long    VBI_LINES
    long    ACT_VBI_LINES
    long    F_ONE
    long    RB_MIN_VBI
    
    long    buffer[64]

OBJ
    debug : "FullDuplexSerial"
    f : "FloatMath"
    fs: "FloatString"

PUB tst
    debug.start(31, 30, 0, 115200)
    waitcnt(cnt + clkfreq * 4)
    calculate(1024, 768, 60, 0)
    dump

PUB getModeDescriptor
    return @buffer

PUB calculate(h, v, rate, rb) : i | scale

    F_ONE := f.FFloat(1)
    
    V_SYNC_RND := 10
    if h * 3 == v * 4
        V_SYNC_RND := 4
    if h * 9 == v * 16
        V_SYNC_RND := 5
    if h * 10 == v * 16
        V_SYNC_RND := 6
    if h * 4 == v * 5
        V_SYNC_RND := 7
    if h * 9 == v * 15
        V_SYNC_RND := 7

    V_FIELD_RATE_RQD := rate
    H_PIXELS_RND := h & !7
    TOTAL_ACTIVE_PIXELS := H_PIXELS_RND
    V_LINES_RND := v & !7
    if rb
        calculateRB
    else
        calculateCRT
        
    i := 0
    buffer[i++] := H_PIXELS_RND
    buffer[i++] := V_LINES_RND
    buffer[i++] := ACT_PIXEL_FREQ
    scale := 16
    if (H_PIXELS_RND & 15) OR (H_SYNC & 15) OR (H_BP & 15) OR (H_FP & 15)
        scale := 8
    buffer[i++] := scale
    buffer[i++] := H_PIXELS_RND / scale
    buffer[i++] := H_FP / scale
    buffer[i++] := H_SYNC / scale
    buffer[i++] := H_BP / scale
    buffer[i++] := 3
    buffer[i++] := V_SYNC_BP - V_BACK_PORCH
    buffer[i++] := V_BACK_PORCH
    if rb
        buffer[i++] := 0
        buffer[i++] := 1
    else
        buffer[i++] := 1
        buffer[i++] := 0
    

PRI calculateCRT : x | y
    H_PERIOD_EST := f.FSub(f.FDiv(F_ONE, f.FFloat(V_FIELD_RATE_RQD)), f.FDiv(f.FFloat(MIN_VSYNC_BP), f.FFloat(1000000)))
    H_PERIOD_EST := f.FDiv(H_PERIOD_EST, f.FFloat(V_LINES_RND + MIN_V_PORCH_RND + INTERLACE))
    
    V_SYNC_BP := f.FTrunc(f.fDiv(f.FDiv(f.FFloat(MIN_VSYNC_BP), f.FFloat(1000000)), H_PERIOD_EST)) + 1
    if V_SYNC_BP < V_SYNC_RND + MIN_V_BPORCH
       V_SYNC_BP := V_SYNC_RND + MIN_V_BPORCH
    V_BACK_PORCH := V_SYNC_BP - V_SYNC_RND
    TOTAL_V_LINES := V_LINES_RND + V_SYNC_BP + MIN_V_PORCH_RND
    IDEAL_DUTY_CYCLE := f.FSub(f.FFloat(C_PRIME) , f.FMul(f.FFloat(M_PRIME) , f.fMul(H_PERIOD_EST, f.FFloat(1000))))
    
    if f.FTrunc(IDEAL_DUTY_CYCLE) < 20
        x := f.FFloat(20)
        y := f.FFloat(80)
    else
        x := IDEAL_DUTY_CYCLE
        y := f.FSub(f.FFloat(100), x)
    H_BLANK :=  f.FMul(f.FFloat(TOTAL_ACTIVE_PIXELS), x)
    H_BLANK :=  f.FDiv(H_BLANK, y)
    H_BLANK :=  f.FDiv(H_BLANK, f.FFloat(2 * CELL_GRAN_RND))
    H_BLANK :=  f.FTrunc(H_BLANK)
    H_BLANK :=  H_BLANK * (2 * CELL_GRAN_RND)
    TOTAL_PIXELS := TOTAL_ACTIVE_PIXELS + H_BLANK
    ACT_PIXEL_FREQ := CLOCK_STEP * f.FTrunc(             f.FDiv(              f.FDiv(f.FFloat(TOTAL_PIXELS), H_PERIOD_EST),f.FFloat(CLOCK_STEP)          ))
    ACT_H_FREQ := ACT_PIXEL_FREQ / TOTAL_PIXELS
    ACT_FIELD_RATE := ACT_PIXEL_FREQ / TOTAL_PIXELS * 1000 / TOTAL_V_LINES

    H_SYNC := (TOTAL_PIXELS * 8 / 100) & !7
    H_BP := H_BLANK >> 1
    H_FP := H_BLANK - H_SYNC - H_BP


PRI calculateRB : x | y
    H_PERIOD_EST := f.FSub(f.FDiv(F_ONE, f.FFloat(V_FIELD_RATE_RQD)), f.FDiv(f.FFloat(RB_MIN_V_BLANK), f.FFloat(1000000)))
    H_PERIOD_EST := f.FDiv(H_PERIOD_EST, f.FFloat(V_LINES_RND))
    
    VBI_LINES := f.FTrunc(f.FDiv(f.FDiv(f.FFloat(RB_MIN_V_BLANK), f.FFloat(1000000)), H_PERIOD_EST)) + 1
    RB_MIN_VBI := RB_V_FPORCH+V_SYNC_RND+MIN_V_BPORCH
    ACT_VBI_LINES := VBI_LINES
    if VBI_LINES < RB_MIN_VBI
        ACT_VBI_LINES := RB_MIN_VBI
    TOTAL_V_LINES := ACT_VBI_LINES + V_LINES_RND
    TOTAL_PIXELS := RB_H_BLANK + TOTAL_ACTIVE_PIXELS
    ACT_PIXEL_FREQ := CLOCK_STEP * f.FRound(   f.FDiv(f.FFloat(V_FIELD_RATE_RQD * TOTAL_V_LINES * TOTAL_PIXELS), f.fFloat(CLOCK_STEP))    )
    ACT_H_FREQ := f.fRound(f.FDiv(f.FFloat(ACT_PIXEL_FREQ) , f.FFloat(TOTAL_PIXELS)))
    ACT_FIELD_RATE := ACT_PIXEL_FREQ / TOTAL_PIXELS * 1000 / TOTAL_V_LINES
    H_SYNC := 32
    H_BP := 160 >> 1
    H_FP := 160 - H_SYNC - H_BP
    
    V_SYNC_BP := ACT_VBI_LINES - RB_V_FPORCH
    V_BACK_PORCH := V_SYNC_BP - V_SYNC_RND


PRI dump : i
    debug.str(string("H_PERIOD_EST = "))
    debug.str(fs.floatToString(H_PERIOD_EST))
    debug.tx($0A)
    debug.str(string("VBI_LINES    = "))
    debug.dec(VBI_LINES)
    debug.tx($0A)
    debug.str(string("RB_MIN_VBI    = "))
    debug.dec(RB_MIN_VBI)
    debug.tx($0A)
    debug.str(string("TOTAL_V_LINES    = "))
    debug.dec(TOTAL_V_LINES)
    debug.tx($0A)
    
    debug.str(string("V_SYNC_BP    = "))
    debug.dec(V_SYNC_BP)
    debug.tx($0A)
    debug.str(string("V_BACK_PORCH = "))
    debug.dec(V_BACK_PORCH)
    debug.tx($0A)
    debug.str(string("IDEAL_DUTY_CYCLE= "))
    debug.str(fs.floatToString(IDEAL_DUTY_CYCLE))
    debug.tx($0A)
    debug.str(string("H_BLANK = "))
    debug.dec(H_BLANK)
    debug.tx($0A)
    debug.str(string("TOTAL_PIXELS = "))
    debug.dec(TOTAL_PIXELS)
    debug.tx($0A)
    debug.str(string("ACT_PIXEL_FREQ = "))
    debug.dec(ACT_PIXEL_FREQ)
    debug.tx($0A)
    debug.str(string("ACT_H_FREQ = "))
    debug.dec(ACT_H_FREQ)
    debug.tx($0A)
    debug.str(string("ACT_FIELD_RATE = "))
    debug.dec(ACT_FIELD_RATE)
    debug.tx($0A)

    debug.tx($0A)
    debug.tx($0A)

    i := 0
    repeat 13
        debug.dec(buffer[i++])
        debug.tx($0A)
    