#define DEBUG
CON
    _clkmode = xtal1+pll16x
    _clkfreq = 80_000_000

    #0, OFF_W, OFF_H,OFF_PIXELCLOCK, OFF_HSYNC_SCALE, OFF_HVIS, OFF_HFP
        OFF_HSYNC, OFF_HBP, OFF_VFP, OFF_VSYNC, OFF_VBP, OFF_VPOL, OFF_HPOL

    I_MOV       = %101000_0010_1111 << 18
    I_MOVI      = %010110_0010_1111 << 18
    I_MOVS      = %010100_0010_1111 << 18
    I_MOVD      = %010101_0010_1111 << 18
    I_DJNZ      = %111001_0011_1111 << 18
    I_WAITVID   = %111111_0000_1111 << 18
    I_JMP       = %010111_0001_1111 << 18
    I_SHL       = %001011_0010_1111 << 18
    I_ADD       = %100000_0010_1111 << 18

    
    IMM         = |< 22
    
    REG_VSCL    = $1FF
    REG_VCFG    = $1FE
    REG_PHSA    = $1FC
    REG_CTRA    = $1F8
    REG_FRQA    = $1FA
    REG_DIRA    = $1F6
    
    POOL_RESERVE = 40

OBJ
#ifdef DEBUG
    debug : "FullDuplexSerial"
#endif
    bld : "patternbuilder"
VAR
    long    pc
    long    poolSize
    long    cog


PUB startPattern(mode, buf)
#ifdef DEBUG
    debug.start(31, 30, 0, 115200)
    waitcnt(CNT + CLKFREQ * 3)
#endif
    if cog
        cogstop(cog-1)
    build(mode, buf)
    cog := cognew(@driver, 0) + 1
#ifdef DEBUG
    disassemble(pc)
#endif

PUB build(mode, pattern) | loopAddress
    calcFRQA(long[mode][OFF_PIXELCLOCK])
    poolSize := 0
    pc := (@scratchpad - @driver) >> 2
    loopAddress := pc
    buildVSync(mode)
    buildVBP(mode)
    buildPattern(mode, pattern)
    buildVFP(mode)
    emit(I_JMP, 0, loopAddress | IMM) 


PRI buildVSync(mode) : vsyncLoop | color
    if long[mode][OFF_VSYNC] > 1
        emit(I_MOV, coga(@group_counter), allocate(long[mode][OFF_VSYNC], 1))
        vsyncLoop := pc
    color := blankColor(mode, true, true, true)
    buildPulse(color, long[mode][OFF_HSYNC_SCALE], long[mode][OFF_HSYNC])
    color := blankColor(mode, false, true, true)
    buildPulse(color, long[mode][OFF_HSYNC_SCALE], long[mode][OFF_HBP])
    buildPulse(color, long[mode][OFF_HSYNC_SCALE], long[mode][OFF_HVIS])
    buildPulse(color, long[mode][OFF_HSYNC_SCALE], long[mode][OFF_HFP])
    if long[mode][OFF_VSYNC] > 1
        emit(I_DJNZ, coga(@group_counter), vsyncLoop | IMM)

PRI buildVBP(mode) : loop | color
    if long[mode][OFF_VBP] > 1
        emit(I_MOV, coga(@group_counter), allocate(long[mode][OFF_VBP],1))
        loop := pc
    color := blankColor(mode, true, false, true)
    buildPulse(color, long[mode][OFF_HSYNC_SCALE], long[mode][OFF_HSYNC])
    color := blankColor(mode, false, false, true)
    buildPulse(color, long[mode][OFF_HSYNC_SCALE], long[mode][OFF_HBP])
    buildPulse(color, long[mode][OFF_HSYNC_SCALE], long[mode][OFF_HVIS])
    buildPulse(color, long[mode][OFF_HSYNC_SCALE], long[mode][OFF_HFP])
    if long[mode][OFF_VSYNC] > 1
        emit(I_DJNZ, coga(@group_counter), loop | IMM)

PRI buildVFP(mode) : loop | color
    if long[mode][OFF_VBP] > 1
        emit(I_MOV, coga(@group_counter), allocate(long[mode][OFF_VFP], 1))
        loop := pc
    color := blankColor(mode, true, false, true)
    buildPulse(color, long[mode][OFF_HSYNC_SCALE], long[mode][OFF_HSYNC])
    color := blankColor(mode, false, false, true)
    buildPulse(color, long[mode][OFF_HSYNC_SCALE], long[mode][OFF_HBP])
    buildPulse(color, long[mode][OFF_HSYNC_SCALE], long[mode][OFF_HVIS])
    buildPulse(color, long[mode][OFF_HSYNC_SCALE], long[mode][OFF_HFP])
    if long[mode][OFF_VSYNC] > 1
        emit(I_DJNZ, coga(@group_counter), loop | IMM)

PRI emit(instr, dst, src)
    long[@driver][pc] := instr | (dst << 9) | src
    pc++

PRI coga(o)
    return (o - @driver) >> 2

PRI blankColor(mode, hs, vs, pl) : color
    ifnot long[mode][OFF_VPOL]
        color |= 1
    ifnot long[mode][OFF_HPOL]
        color |= 2
    if vs
        color ^= 1
    if hs
        color ^= 2
    if pl
        color := allocate(color, 0)

PRI buildPulse(color, scale, ticks) : vsc
    vsc := scale << 12
    vsc += scale * ticks
    emit(I_MOV,     REG_VSCL,   allocate(vsc, 1))
    emit(I_WAITVID, color,      0 | IMM)

PRI buildPattern(mode, pattern) | nGroups, rep, loop
    nGroups := long[pattern]
    'pr(string("buildPattern"), string("nGroups"), nGroups)
    pattern+= 4
    repeat nGroups
        rep := long[pattern]
        'pr(string("buildPattern"), string("rep"), rep)
        pattern+= 4
        if rep > 1
            emit(I_MOV, coga(@group_counter), allocate(rep, 1))
            loop := pc
        pattern += buildGroup(mode, pattern)
        if rep > 1
            emit(I_DJNZ, coga(@group_counter), loop | IMM)

PRI buildGroup(mode, grp) : start | nLines, rep, loop
    start := grp
    nLines := long[grp]
    grp += 4
    'pr(string("buildGroup"), string("nLines"), nLines)
    repeat nLines
        rep := long[grp]
        grp += 4
        'pr(string("buildGroup"), string("rep"), rep)
        if rep > 1
            emit(I_MOV, coga(@line_counter), allocate(rep, 1))
            loop := pc
        grp += buildLine(mode, grp)
        if rep > 1
            emit(I_DJNZ, coga(@line_counter), loop | IMM)
    return grp - start

PRI buildLine(mode, ln) : start | nSections, rep, loop, color
    'emit(I_ADD, REG_PHSA, allocate(819, 1))
    start := ln
    color := blankColor(mode, true, false, true)
    buildPulse(color, long[mode][OFF_HSYNC_SCALE], long[mode][OFF_HSYNC])
    color := blankColor(mode, false, false, true)
    buildPulse(color, long[mode][OFF_HSYNC_SCALE], long[mode][OFF_HBP])
    nSections := long[ln]
    ln += 4
    'pr(string("buildLine"), string("nSections"), nSections)
    repeat nSections
        rep := long[ln]
        ln += 4
        if rep > 1
            emit(I_MOV, coga(@section_counter), allocate(rep, 1))
            loop := pc
        ln += buildSection(mode, ln)
        if rep > 1
            emit(I_DJNZ, coga(@section_counter), loop | IMM)
    buildPulse(color, long[mode][OFF_HSYNC_SCALE], long[mode][OFF_HFP])
    return ln - start

PRI buildSection(mode, sec) : start| colors, pixels, i, colors_values, scale, total, vsc
    start := sec
    colors := long[sec]
    sec += 4
    pixels := long[sec]
    sec += 4
    scale := long[sec]
    sec += 4
    total := long[sec]
    sec += 4
    colors_values := (colors & $FC) | blankColor(mode, false, false, false)
    colors_values |= (((colors >> 8) & $FC) | blankColor(mode, false, false, false)) << 8
    vsc := (scale << 12) + total
#ifdef DEBUG
    debug.str(string("Pixels "))
    debug.hex(pixels, 8)
    debug.tx($0A)
    debug.str(string("Colors "))
    debug.hex(colors_values, 8)
    debug.tx($0A)
    debug.str(string("VSCL "))
    debug.hex(vsc, 8)
    debug.tx($0A)
#endif
    emit(I_MOV,     REG_VSCL, allocate(vsc, 1))
    emit(I_WAITVID, allocate(colors_values, 0), allocate(pixels, 1))
    return sec - start

PRI allocate(value, immediate) | i
    if immediate  <> 0
        if value < 512
            if value => 0
                return value | IMM
    i := 0
    if poolSize
        repeat poolSize
            if long[@pool][i] == value
                return i + coga(@pool)
            i++
    long[@pool][poolSize] := value
    result := poolSize++ + coga(@pool)
  
PRI calcFRQA(pixelclock) | prescaler, fc, weight, base
    frqa_0 := 0
    prescaler := 4
    repeat while (prescaler > 1) AND (prescaler * pixelclock   > 128_000_000)
        prescaler >>= 1
    ctr_mode := (1 << 3) | (8 - (>| prescaler))
    fc := (pixelclock >> 4) * prescaler
    weight := 1 << 30
    base := CLKFREQ >> 2
    repeat while fc > 0
        repeat while fc < base
            base >>= 1
            weight >>= 1
        fc -= base
        frqa_0 += weight
    frqa_0 &= !15

DAT

driver
                    ORG 0
                    mov     FRQA, frqa_0
                    movi    VCFG, # 1 << 6  ' | 1 << 5
                    movs    VCFG, #$FF
                    movd    VCFG, #%010
                    mov     DIRA, #$FF
                    shl     DIRA, #16
                    movi    CTRA, ctr_mode
                    jmp     #scratchpad
frqa_0              long    0
ctr_mode            long    1 << 3 | %101
group_counter       long    0
line_counter        long    0
section_counter     long    0

scratchpad
                    long    0[496-POOL_RESERVE-$]
pool                long    0[POOL_RESERVE]
                    FIT     496
                    
test_mode
                    long    640         ' width
                    long    480         ' height
                    long    25175000    ' pixelclock
                    long    16          ' hsync_scale
                    long    40          ' hvis
                    long    1           ' hfp
                    long    6           ' hsync
                    long    3           'hbp
                    long    10          'vfp
                    long    2           'vsync
                    long    33          'vbp
                    long    0           'vpol
                    long    0           'hpol
                    
test_mode2
                    long    640          ' width
                    long    1024          ' height
                    long    94750000    ' pixelclock
                    long    4           ' hsync_scale
                    long    160          ' hvis
                    long    12            ' hfp
                    long    17          ' hsync
                    long    29           'hbp
                    long    3           'vfp
                    long    7          'vsync
                    long    53          'vbp
                    long    1           'vpol
                    long    0           'hpol
                    
test_pattern
                    long    2           ' nGroups
                    
        '' ------------- Group 1 ------------------------------
                    long    1           ' rep
                    long    1           ' nLines
        '' -------------  Line 1.1 ----------------------------
                    long    240         ' rep
                    long    1           ' nSections
        '' --------------  Section 1.1.1 ----------------------
                    long    1           ' rep
                    long    $AA         ' colors
                    long    0           ' first
                    long    1           ' scale
                    long    640         'total
        '' ------------- Group 2 ------------------------------
                    long    1           ' rep
                    long    1           ' nLines
        '' -------------  Line 2.1 ----------------------------
                    long    240         ' rep
                    long    1           ' nSections
        '' --------------  Section 2.1.1 ----------------------
                    long    1           ' rep
                    long    $FF         ' colors
                    long    0           ' first
                    long    1           ' scale
                    long    640         'total

#ifdef DEBUG
PRI disassemble(m) | addr, instr, p, txt, has_dst, has_src
    addr := @driver
    p := 0
    repeat m
        instr := long[addr]
        addr += 4
        has_dst := True
        has_src := True
        case instr & ((%111111_1111_1111 << 18) & !IMM)
            I_MOV & !IMM       : txt := string("mov     ")
            I_MOVI & !IMM      : txt := string("movi    ")
            I_MOVS & !IMM      : txt := string("movs    ")
            I_MOVD & !IMM      : txt := string("movd    ")
            I_SHL  & !IMM      : txt := string("shl     ")
            I_DJNZ & !IMM      : txt := string("djnz    ")
            I_WAITVID & !IMM   : txt := string("waitvid ")
            I_JMP & !IMM       : 
                txt := string("jmp     ")
                has_dst := False
            other :
                txt := string("        ")
                has_dst := False
                has_src := False
        debug.str(string("l_"))
        debug.hex(p++, 3)
        debug.tx(" ")
        debug.str(txt)
        if (has_dst)
            maybeReg((instr >> 9) & $1FF)
        else
            if has_src
                debug.str(string("      "))
        if has_src
            if instr & IMM
                debug.tx("#")
            else
                debug.tx(" ")
            debug.tx("$")
            debug.hex(instr & $1FF, 3)
        else
            if has_dst
                debug.str(string("     "))
        if has_dst OR has_src
            debug.str(string("   '   LONG $"))
        else
            debug.str(string("                  LONG $"))
        debug.hex(instr, 8)
        debug.tx($0A)
    addr := 0
    debug.str(string("             LONG 0[$"))
    debug.hex((@pool - @driver) >> 2 - m, 3)
    debug.tx("]")
    debug.tx($0A)
    debug.str(string("'' Constatnts pool", $0A))
    debug.str(string("             ORG $"))
    debug.hex((@pool - @driver) >> 2, 3)
    debug.tx($0A)
    repeat 496 - (@pool - @driver) >> 2
        debug.str(string("L_"))
        debug.hex(addr + ((@pool - @driver)>>2), 3)
        debug.str(string("         LONG $"))
        debug.hex(pool[addr++], 8)
        debug.tx($0A)
        

PRI dump : i
    i := @driver
    repeat while i < @test_mode
        debug.bin(long[i], 32)
        debug.tx($0A)
        i += 4

PRI maybeReg(x) | s
    s := 0
    case x
        REG_VSCL: s := string("VSCL")
        REG_VCFG: s := string("VCFG")
        REG_PHSA: s := string("PHSA")
        REG_CTRA: s := string("CTRA")
        REG_FRQA: s := string("FRQA")
        REG_DIRA: s := string("DIRA")
    if s
        debug.str(s)
    else
        debug.tx("$")
        debug.hex(x, 3)
    debug.tx(",")
    debug.tx(" ")
    

#endif
