CON
    OFF_W = 0
    OFF_H = 1
    
    NUM_PATTERNS = 27

VAR
    long    currentPattern

PUB nextPattern
    currentPattern++
    if  (currentPattern < 0) OR (currentPattern => NUM_PATTERNS)
        currentPattern := 0

PUB build(mode, buffer)
    CASE currentPattern
        0:      buildWhiteField(mode, buffer)
        1:      buildRedField(mode, buffer)
        2:      buildGreenField(mode, buffer)
        3:      buildBlueField(mode, buffer)
        4:      buildBlock4x3(mode, buffer)
        5:      buildBlock16x12(mode, buffer)
        6:      buildVLines(mode, buffer)
        7:      buildGrid4x3Red(mode, buffer)
        8:      buildGrid4x3Green(mode, buffer)
        9:      buildGrid4x3Blue(mode, buffer)
        10:     buildGrid4x3White(mode, buffer)
        11:     buildGrid16x12Red(mode, buffer)
        12:     buildGrid16x12Green(mode, buffer)
        13:     buildGrid16x12Blue(mode, buffer)
        14:     buildGrid16x12White(mode, buffer)
        15:     buildColorBars8(mode, buffer)
        16:     buildColorBars16(mode, buffer)
        17:     buildColorGrid(mode, buffer)
        18:     buildWindows1(mode, buffer)
        19:     buildWindows2(mode, buffer)
        20:     buildFlag(mode, buffer)
        21:     buildVLines2(mode, buffer)
        22:     buildVLines4(mode, buffer)
        23:     buildVLines8(mode, buffer)
        24:     buildVLines16(mode, buffer)
        25:     buildVLines32(mode, buffer)
        26:     buildHLines2(mode, buffer)

    
    
PUB buildWhiteField(mode, buffer)
    buildField(mode, buffer, $FC)

PUB buildRedField(mode, buffer)
    buildField(mode, buffer, $C0)

PUB buildGreenField(mode, buffer)
    buildField(mode, buffer, $30)

PUB buildBlueField(mode, buffer)
    buildField(mode, buffer, $0C)

PUB buildBlock4x3(mode, buffer) : i | color
    i := 0
    color := $FF
    long[buffer][i++] := 3     ' 3 groups
    repeat 3
        '' ==== build group
        long [buffer][i++] := 1     ' repeat 1 time
        long[buffer][i++] := 1     ' 1 line
        '' ======== build line
        long[buffer][i++] := long[mode][OFF_H] / 3 ' repeat HEIGHT/3 times
        long[buffer][i++] := 4                 ' 4 sections
        repeat 4
            '' =========== build section
            long[buffer][i++] := 1                 ' repeat 1 time
            long[buffer][i++] := color             '
            long[buffer][i++] := 0                 ' first - not used
            long[buffer][i++] := 1                 ' scale
            long[buffer][i++] := long[mode][OFF_W] / 4 ' total
            color ^= $FF
        color ^= $FF

PUB buildBlock16x12(mode, buffer) : i | color, rest
    i := 0
    color := $FF
    rest := long[mode][OFF_H] // 12
    long[buffer][i++] := 1     ' 1 group
    if rest
        long[buffer][i-1] := 2
    '' ==== build group
    long [buffer][i++] := 6 ' repeat
    long[buffer][i++] := 2     ' 2 lines
    repeat 2
        '' ======== build line
        long[buffer][i++] := long[mode][OFF_H] / 12 ' repeat
        long[buffer][i++] := 16                 ' 4 sections
        repeat 16
            '' =========== build section
            long[buffer][i++] := 1                 ' repeat 1 time
            long[buffer][i++] := color             '
            long[buffer][i++] := 0                 ' first - not used
            long[buffer][i++] := 1                 ' scale
            long[buffer][i++] := long[mode][OFF_W] / 16 ' total
            color ^= $FF
        color ^= $FF
    if rest
        '' ==== build group
        long [buffer][i++] := 1 ' repeat
        long[buffer][i++] := 1     ' 1 line
        '' ======== build line
        long[buffer][i++] := rest ' repeat
        long[buffer][i++] := 16                 ' 4 sections
        repeat 16
            '' =========== build section
            long[buffer][i++] := 1                 ' repeat 1 time
            long[buffer][i++] := color             '
            long[buffer][i++] := 0                 ' first - not used
            long[buffer][i++] := 1                 ' scale
            long[buffer][i++] := long[mode][OFF_W] / 16 ' total
            color ^= $FF
        color ^= $FF

PUB buildVLines(mode, buffer) : i
    i := 0
    long[buffer][i++] := 1     ' 1 group
    '' ==== build group
    long[buffer][i++] := 1     ' repeat 1 time
    long[buffer][i++] := 1     ' 1 line
    '' ======== build line
    long[buffer][i++] := long[mode][OFF_H] ' repeat HEIGHT times
    long[buffer][i++] := 1                 ' 1 section
    '' =========== build section
    long[buffer][i++] := long[mode][OFF_W] >> 5 ' repeat 16
    long[buffer][i++] := $0000_FF_00       ' colors
    long[buffer][i++] := 1                 ' pixels
    long[buffer][i++] := 1                 ' scale
    long[buffer][i++] := 32                ' total

PUB buildGrid4x3Red(mode, buffer)
    buildGrid4x3(mode, buffer, $C0)

PUB buildGrid4x3Green(mode, buffer)
    buildGrid4x3(mode, buffer, $30)

PUB buildGrid4x3Blue(mode, buffer)
    buildGrid4x3(mode, buffer, $0C)

PUB buildGrid4x3White(mode, buffer)
    buildGrid4x3(mode, buffer, $FC)

PUB buildGrid16x12Red(mode, buffer)
    buildGrid16x12(mode, buffer, $C0)

PUB buildGrid16x12Green(mode, buffer)
    buildGrid16x12(mode, buffer, $30)

PUB buildGrid16x12Blue(mode, buffer)
    buildGrid16x12(mode, buffer, $0C)

PUB buildGrid16x12White(mode, buffer)
    buildGrid16x12(mode, buffer, $FC)

PUB buildColorBars8(mode, buffer) : i | c, color
    c := 7
    i := 0
    long[buffer][i++] := 1     ' 1 group
    '' ==== build group
    long[buffer][i++] := 1     ' repeat 1 time
    long[buffer][i++] := 1     ' 1 line
    '' ======== build line
    long[buffer][i++] := long[mode][OFF_H] ' repeat HEIGHT times
    long[buffer][i++] := 4                 ' 4 sections
    repeat 4
        '' =========== build section
        color := ((c & 1) << 3) | ((c & 2) << 4) | ((c & 4) << 5)
        c--
        color |= (((c & 1) << 3) | ((c & 2) << 4) | ((c & 4) << 5)) << 8
        c--
        long[buffer][i++] := 1                      ' repeat 1
        long[buffer][i++] := color                  ' color
        long[buffer][i++] := !1                     ' pixels
        long[buffer][i++] := long[mode][OFF_W] / 8  ' scale
        long[buffer][i++] := long[mode][OFF_W] / 4  ' total

PUB buildColorBars16(mode, buffer) : i | c, color
    c := 7
    i := 0
    long[buffer][i++] := 1     ' 1 group
    '' ==== build group
    long[buffer][i++] := 1     ' repeat 1 time
    long[buffer][i++] := 1     ' 1 line
    '' ======== build line
    long[buffer][i++] := long[mode][OFF_H] ' repeat HEIGHT times
    long[buffer][i++] := 8                  ' 8 sections
    repeat 8
        '' =========== build section
        color := ((c & 1) << 3) | ((c & 2) << 4) | ((c & 4) << 5)
        c--
        color |= (((c & 1) << 3) | ((c & 2) << 4) | ((c & 4) << 5)) << 8
        c--
        long[buffer][i++] := 1                      ' repeat 1
        long[buffer][i++] := color                  ' color
        long[buffer][i++] := !1                     ' pixels
        long[buffer][i++] := long[mode][OFF_W] / 16 ' scale
        long[buffer][i++] := long[mode][OFF_W] / 8 ' total


PRI buildGrid4x3(mode, buffer, color) : i
    i := 0
    long[buffer][i++] := 1     ' 1 group
    '' ==== build group
    long[buffer][i++] := 3     ' repeat 3 times
    long[buffer][i++] := 2     ' 2 lines
    '' ======== build line
    long[buffer][i++] := long[mode][OFF_H] / 3 - 1 ' repeat HEIGHT/3-1 times
    long[buffer][i++] := 1                 ' 1 section
    '' =========== build section
    long[buffer][i++] := 4                  ' repeat 4
    long[buffer][i++] := color
    long[buffer][i++] := !1                ' pixels
    long[buffer][i++] := 1                 ' scale
    long[buffer][i++] := long[mode][OFF_W]/4 ' total
    '' ======== build line
    long[buffer][i++] := 1                 ' 1 time
    long[buffer][i++] := 1                 ' 1 section
    '' =========== build section
    long[buffer][i++] := 1                  ' repeat 1
    long[buffer][i++] := color
    long[buffer][i++] := 0                 ' pixels
    long[buffer][i++] := 1                 ' scale
    long[buffer][i++] := long[mode][OFF_W] ' total

PRI buildGrid16x12(mode, buffer, color) : i
    i := 0
    long[buffer][i++] := 1     ' 1 group
    '' ==== build group
    long[buffer][i++] := 12     ' repeat 12 times
    long[buffer][i++] := 2     ' 2 lines
    '' ======== build line
    long[buffer][i++] := long[mode][OFF_H] / 12 - 1 ' repeat HEIGHT/12-1 times
    long[buffer][i++] := 1                 ' 1 section
    '' =========== build section
    long[buffer][i++] := 16                  ' repeat 16
    long[buffer][i++] := color             ' colors
    long[buffer][i++] := !1                ' pixels
    long[buffer][i++] := 1                 ' scale
    long[buffer][i++] := long[mode][OFF_W]/16 ' total
    '' ======== build line
    long[buffer][i++] := 1                 ' 1 time
    long[buffer][i++] := 1                 ' 1 section
    '' =========== build section
    long[buffer][i++] := 1                  ' repeat 1
    long[buffer][i++] := color             ' colors
    long[buffer][i++] := 0                 ' pixels
    long[buffer][i++] := 1                 ' scale
    long[buffer][i++] := long[mode][OFF_W] ' total

PUB buildColorGrid(mode, buffer) : i | toph, both, leftw, rightw, c, colors[4]
    colors[0] := $FC
    colors[1] := $C0
    colors[2] := $30
    colors[3] := $0C
    i := 0
    toph := long[mode][OFF_H] / 16 - 1
    both := long[mode][OFF_H] / 8 - toph - 1
    leftw := long[mode][OFF_W] / 16 - 1
    rightw := long[mode][OFF_W] / 8 - leftw - 1
    long[buffer][i++] := 1     ' 1 group
    '' ==== build group
    long[buffer][i++] := 2     ' repeat 2 times
    long[buffer][i++] := 12    ' 12 lines
    c := 0
    repeat 4
            '' ======== build line
            long[buffer][i++] := toph ' repeat toph times
            long[buffer][i++] := 9                 ' 9 sections
            '' =========== build section
            long[buffer][i++] := 1                 ' repeat 1 time
            long[buffer][i++] := 0                 ' colors
            long[buffer][i++] := 0                 ' pixels
            long[buffer][i++] := 1                 ' scale
            long[buffer][i++] := leftw             ' total
            repeat 7
                '' =========== build section
                long[buffer][i++] := 1                 ' repeat 1 time
                long[buffer][i++] := colors[c&3]       ' colors
                long[buffer][i++] := !1                ' pixels
                long[buffer][i++] := 1                 ' scale
                long[buffer][i++] := rightw + leftw + 1 ' total
                c++
            '' =========== build section
            long[buffer][i++] := 1                 ' repeat 1 time
            long[buffer][i++] := colors[c&3]       ' colors
            long[buffer][i++] := !1                ' pixels
            long[buffer][i++] := 1                 ' scale
            long[buffer][i++] := rightw +1          ' total
            c++
            '' ======== build line
            long[buffer][i++] := 1                 ' repeat 1 time
            long[buffer][i++] := 8                 ' 8 sections
            repeat 8
                '' =========== build section
                long[buffer][i++] := 1                 ' repeat 1 time
                long[buffer][i++] := colors[c&3]       ' colors
                long[buffer][i++] := 0                 ' pixels
                long[buffer][i++] := 1                 ' scale
                long[buffer][i++] := leftw + rightw + 1 ' total
                c++

            '' ======== build line
            long[buffer][i++] := both               ' repeat both times
            long[buffer][i++] := 9                 ' 9 sections
            '' =========== build section
            long[buffer][i++] := 1                 ' repeat 1 time
            long[buffer][i++] := 0                 ' colors
            long[buffer][i++] := 0                 ' pixels
            long[buffer][i++] := 1                 ' scale
            long[buffer][i++] := leftw             ' total
            repeat 7
                '' =========== build section
                long[buffer][i++] := 1                 ' repeat 1 time
                long[buffer][i++] := colors[c&3]       ' colors
                long[buffer][i++] := !1                ' pixels
                long[buffer][i++] := 1                 ' scale
                long[buffer][i++] := rightw + leftw + 1 ' total
                c++
            '' =========== build section
            long[buffer][i++] := 1                 ' repeat 1 time
            long[buffer][i++] := colors[c&3]       ' colors
            long[buffer][i++] := !1                 ' pixels
            long[buffer][i++] := 1                 ' scale
            long[buffer][i++] := rightw +1          ' total
            c++
        c++
    
PUB buildWindows1(mode, buffer) : i
    i := 0
    long[buffer][i++] := 1     ' 1 group
    '' ==== build group
    long[buffer][i++] := 1     ' repeat 1 time
    long[buffer][i++] := 2     ' 1 lines
    '' ======== build line
    long[buffer][i++] := long[mode][OFF_H] / 2 ' repeat HEIGHT/2 times
    long[buffer][i++] := 1                 ' 1 section
    '' =========== build section
    long[buffer][i++] := 1                 ' repeat 1 time
    long[buffer][i++] := $00_C0            ' color
    long[buffer][i++] := $0000_FFFF        ' pixels
    long[buffer][i++] := long[mode][OFF_W]/32' scale
    long[buffer][i++] := long[mode][OFF_W]  ' total
    '' ======== build line
    long[buffer][i++] := long[mode][OFF_H] / 2 ' repeat HEIGHT/2 times
    long[buffer][i++] := 1                 ' 1 section
    '' =========== build section
    long[buffer][i++] := 1                 ' repeat 1 time
    long[buffer][i++] := $30_0C            ' color
    long[buffer][i++] := $0000_FFFF        ' pixels
    long[buffer][i++] := long[mode][OFF_W]/32' scale
    long[buffer][i++] := long[mode][OFF_W]  ' total

PUB buildWindows2(mode, buffer) : i
    i := 0
    long[buffer][i++] := 1     ' 1 group
    '' ==== build group
    long[buffer][i++] := 1     ' repeat 1 time
    long[buffer][i++] := 2     ' 2 lines
    '' ======== build line
    long[buffer][i++] := long[mode][OFF_H] / 2 ' repeat HEIGHT/2 times
    long[buffer][i++] := 2                 ' 2 sections
    '' =========== build section
    long[buffer][i++] := 1                 ' repeat 1 time
    long[buffer][i++] := $FC               ' color
    long[buffer][i++] := 0                 ' first - not used
    long[buffer][i++] := 1                 ' scale
    long[buffer][i++] := long[mode][OFF_W]/2 ' total
    '' =========== build section
    long[buffer][i++] := 1                 ' repeat 1 time
    long[buffer][i++] := $C0               ' color
    long[buffer][i++] := 0                 ' first - not used
    long[buffer][i++] := 1                 ' scale
    long[buffer][i++] := long[mode][OFF_W]/2 ' total
    '' ======== build line
    long[buffer][i++] := long[mode][OFF_H] / 2 ' repeat HEIGHT/2 times
    long[buffer][i++] := 2                 ' 2 sections
    '' =========== build section
    long[buffer][i++] := 1                 ' repeat 1 time
    long[buffer][i++] := $30               ' color
    long[buffer][i++] := 0                 ' first - not used
    long[buffer][i++] := 1                 ' scale
    long[buffer][i++] := long[mode][OFF_W]/2 ' total
    '' =========== build section
    long[buffer][i++] := 1                 ' repeat 1 time
    long[buffer][i++] := $0C               ' color
    long[buffer][i++] := 0                 ' first - not used
    long[buffer][i++] := 1                 ' scale
    long[buffer][i++] := long[mode][OFF_W]/2 ' total

PUB buildFlag(mode, buffer) : i
    i := 0
    long[buffer][i++] := 1     ' 1 group
    '' ==== build group
    long[buffer][i++] := 1     ' repeat 1 time
    long[buffer][i++] := 3     ' 3 lines
    '' ======== build line
    long[buffer][i++] := long[mode][OFF_H] / 3 ' repeat HEIGHT/3 times
    long[buffer][i++] := 1                 ' 1 section
    '' =========== build section
    long[buffer][i++] := 1                 ' repeat 1 time
    long[buffer][i++] := $FC               ' color
    long[buffer][i++] := 0                 ' first - not used
    long[buffer][i++] := 1                 ' scale
    long[buffer][i++] := long[mode][OFF_W] ' total
    '' ======== build line
    long[buffer][i++] := long[mode][OFF_H] / 3 ' repeat HEIGHT/3 times
    long[buffer][i++] := 1                 ' 1 section
    '' =========== build section
    long[buffer][i++] := 1                 ' repeat 1 time
    long[buffer][i++] := $0C               ' color
    long[buffer][i++] := 0                 ' first - not used
    long[buffer][i++] := 1                 ' scale
    long[buffer][i++] := long[mode][OFF_W] ' total
    '' ======== build line
    long[buffer][i++] := long[mode][OFF_H] - (long[mode][OFF_H] / 3) * 2 ' repeat HEIGHT/3 times
    long[buffer][i++] := 1                 ' 1 section
    '' =========== build section
    long[buffer][i++] := 1                 ' repeat 1 time
    long[buffer][i++] := $C0               ' color
    long[buffer][i++] := 0                 ' first - not used
    long[buffer][i++] := 1                 ' scale
    long[buffer][i++] := long[mode][OFF_W] ' total

PUB buildVLines2(mode, buffer) : i
    i := 0
    long[buffer][i++] := 1     ' 1 group
    '' ==== build group
    long[buffer][i++] := 1     ' repeat 1 time
    long[buffer][i++] := 1     ' 1 line
    '' ======== build line
    long[buffer][i++] := long[mode][OFF_H]  ' repeat HEIGHT times
    long[buffer][i++] := 1                 ' 1 section
    '' =========== build section
    long[buffer][i++] := long[mode][OFF_W]/32   ' repeat WIDTH/32 times
    long[buffer][i++] := $0000_00FC        ' colors
    long[buffer][i++] := $CCCC_CCCC        ' pixels
    long[buffer][i++] := 1                 ' scale
    long[buffer][i++] := 32 ' total

PUB buildVLines4(mode, buffer) : i
    i := 0
    long[buffer][i++] := 1     ' 1 group
    '' ==== build group
    long[buffer][i++] := 1     ' repeat 1 time
    long[buffer][i++] := 1     ' 1 line
    '' ======== build line
    long[buffer][i++] := long[mode][OFF_H]  ' repeat HEIGHT times
    long[buffer][i++] := 1                 ' 1 section
    '' =========== build section
    long[buffer][i++] := long[mode][OFF_W]/32   ' repeat WIDTH/32 times
    long[buffer][i++] := $8000_00FC        ' colors
    long[buffer][i++] := $F0F0_F0F0        ' pixels
    long[buffer][i++] := 1                 ' scale
    long[buffer][i++] := 32 ' total

PUB buildVLines8(mode, buffer) : i
    i := 0
    long[buffer][i++] := 1     ' 1 group
    '' ==== build group
    long[buffer][i++] := 1     ' repeat 1 time
    long[buffer][i++] := 1     ' 1 line
    '' ======== build line
    long[buffer][i++] := long[mode][OFF_H]  ' repeat HEIGHT times
    long[buffer][i++] := 1                 ' 1 section
    '' =========== build section
    long[buffer][i++] := long[mode][OFF_W]/32   ' repeat WIDTH/32 times
    long[buffer][i++] := $8000_00FC        ' colors
    long[buffer][i++] := $FF00_FF00        ' pixels
    long[buffer][i++] := 1                 ' scale
    long[buffer][i++] := 32 ' total

PUB buildVLines16(mode, buffer) : i
    i := 0
    long[buffer][i++] := 1     ' 1 group
    '' ==== build group
    long[buffer][i++] := 1     ' repeat 1 time
    long[buffer][i++] := 1     ' 1 line
    '' ======== build line
    long[buffer][i++] := long[mode][OFF_H]  ' repeat HEIGHT times
    long[buffer][i++] := 1                 ' 1 section
    '' =========== build section
    long[buffer][i++] := long[mode][OFF_W]/32   ' repeat WIDTH/32 times
    long[buffer][i++] := $8000_00FC        ' colors
    long[buffer][i++] := $FFFF_0000        ' pixels
    long[buffer][i++] := 1                 ' scale
    long[buffer][i++] := 32 ' total

PUB buildVLines32(mode, buffer) : i | rest
    rest := long[mode][OFF_W] // 64
    i := 0
    long[buffer][i++] := 1     ' 1 group
    '' ==== build group
    long[buffer][i++] := 1     ' repeat 1 time
    long[buffer][i++] := 1     ' 1 line
    '' ======== build line
    long[buffer][i++] := long[mode][OFF_H]  ' repeat HEIGHT times
    long[buffer][i++] := 1                 ' 1 section
    if rest
        long[buffer][i-1] := 2
    '' =========== build section
    long[buffer][i++] := long[mode][OFF_W]/64   ' repeat WIDTH/64 times
    long[buffer][i++] := $0000_00FC        ' colors
    long[buffer][i++] := 1                 ' first - 1
    long[buffer][i++] := 32                ' scale
    long[buffer][i++] := 64 ' total
    if rest
        '' =========== build section
        long[buffer][i++] := 1                 ' repeat 1 time
        long[buffer][i++] := $0000_0000        ' colors
        long[buffer][i++] := 0                 ' pixels
        long[buffer][i++] := 1                 ' scale
        long[buffer][i++] := rest                ' total

PUB buildHLines2(mode, buffer) : i
    i := 0
    long[buffer][i++] := 1     ' 1 group
    '' ==== build group
    long[buffer][i++] := long[mode][OFF_H]/4     ' repeat HEIGHT/4 time
    long[buffer][i++] := 2     ' 2 lines
    '' ======== build line
    long[buffer][i++] := 2          ' repeat 2 times
    long[buffer][i++] := 1                 ' 1 section
    '' =========== build section
    long[buffer][i++] := 1                 ' repeat 1 time
    long[buffer][i++] := $FC        ' colors
    long[buffer][i++] := 0          ' first
    long[buffer][i++] := 1          ' scale
    long[buffer][i++] := long[mode][OFF_W] ' total
    '' ======== build line
    long[buffer][i++] := 2          ' repeat 2 time
    long[buffer][i++] := 1                 ' 1 section
    '' =========== build section
    long[buffer][i++] := 1                 ' repeat 1 time
    long[buffer][i++] := $00        ' colors
    long[buffer][i++] := 0          ' first
    long[buffer][i++] := 1          ' scale
    long[buffer][i++] := long[mode][OFF_W] ' total

PRI buildField(mode, buffer, color) : i
    i := 0
    long[buffer][i++] := 1     ' 1 group
    '' ==== build group
    long[buffer][i++] := 1     ' repeat 1 time
    long[buffer][i++] := 1     ' 1 line
    '' ======== build line
    long[buffer][i++] := long[mode][OFF_H] ' repeat HEIGHT times
    long[buffer][i++] := 1                 ' 1 section
    '' =========== build section
    long[buffer][i++] := 1                 ' repeat 1 time
    long[buffer][i++] := color             ' 
    long[buffer][i++] := 0                 ' first - not used
    long[buffer][i++] := 1                 ' scale
    long[buffer][i++] := long[mode][OFF_W] ' total
